﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectInput;

namespace HaloMap
{
    public class FPCamera
    {
        public Vector3 m_position; // camera position
	    public float m_yaw = 0f;  // rotation around the y axis
	    public float m_pitch = 0f; // rotation around the x axis
	    public float m_roll = 0f; // rotation around the z axis
	    public Vector3 m_up, m_look, m_right; // camera axis

        public FPCamera(Vector3 startPos)
        {
            m_position = startPos;
	        // Start with an orthagonal camera axis
	        m_up = new Vector3(0.0f, 1.0f, 0.0f);
	        m_look = new Vector3(0.0f, 0.0f, 1.0f);
	        m_right = new Vector3(1.0f, 0.0f, 0.0f);
        }

        public void CalculateViewMatrix(Matrix viewMatrix)
        {
	        /* Start with our camera axis pointing down z
	         An alternative method is to just keep adjusting our axis but if we do that the
	         axis can start to lose its orthogonal shape (due to floating point innacuracies).
	         This could be solved by rebuilding the orthogonal shape each time with the following:
	         1. normalising the look vector
	         2. creating the up vector from the cross product of the look and the right
	         3. normalising up
	         4. creating the right vector from the cross product of the look and the up
	         5. normalising right
	        */

	        m_up = new Vector3(0.0f, 1.0f, 0.0f);
	        m_look = new Vector3(0.0f, 0.0f, 1.0f);
	        m_right = new Vector3(1.0f, 0.0f, 0.0f);

	        // Yaw is rotation around the y axis (m_up)
	        // Create a matrix that can carry out this rotation
	        //Matrix yawMatrix;
            //Matrix.(yawMatrix, m_yaw);
	        // To apply yaw we rotate the m_look & m_right vectors about the m_up vector (using our yaw matrix)
	        //Vector3.TransformCoordinate(m_look, yawMatrix); 
	        //Vector3.TransformCoordinate(m_right, yawMatrix); 

	        // Pitch is rotation around the x axis (m_right)
	        // Create a matrix that can carry out this rotation
	        //Matrix pitchMatrix;
	        //Matrix.RotationAxis(pitchMatrix, m_pitch);
	        // To apply pitch we rotate the m_look and m_up vectors about the m_right vector (using our pitch matrix)
	        //Vector3.TransformCoordinate(m_look, pitchMatrix); 
	        //Vector3.TransformCoordinate(m_up, pitchMatrix); 
        		
	        // Roll is rotation around the z axis (m_look)
	        // Create a matrix that can carry out this rotation
	        //Matrix rollMatrix;
	        //Matrix.RotationAxis(viewMatrix,  m_roll);
	        // To apply roll we rotate up and right about the look vector (using our roll matrix)
	        // Note: roll only really applies for things like aircraft unless you are implementing lean
	        //Vector3.TransformCoordinate(m_right, rollMatrix); 
	        //Vector3.TransformCoordinate(m_up, rollMatrix); 
        	
	        // Build the view matrix from the transformed camera axis
	        //Matrix.RotationYawPitchRoll
            viewMatrix.RotateYawPitchRoll(m_yaw, m_pitch, m_roll);
	        viewMatrix.M11 = m_right.X; viewMatrix.M12 = m_up.X; viewMatrix.M13 = m_look.X;
	        viewMatrix.M21 = m_right.Y; viewMatrix.M22 = m_up.Y; viewMatrix.M23 = m_look.Y;
	        viewMatrix.M31 = m_right.Z; viewMatrix.M32 = m_up.Z; viewMatrix.M33 = m_look.Z;
        	
	        viewMatrix.M41 = - Vector3.Dot(m_position, m_right); 
	        viewMatrix.M42 = - Vector3.Dot(m_position, m_up);
	        viewMatrix.M43 = - Vector3.Dot(m_position, m_look);
        }

        // Move operations
        public void MoveForward(float amount) { m_position += m_look * amount; }
        public void MoveRight(float amount) { m_position += m_right * amount; }
        public void MoveUp(float amount) { m_position += m_up * amount; }

        // Yaw - rotation around y axis
        public void Yaw(float amount) 
        {
	        m_yaw += amount;
	        m_yaw = RestrictAngleTo360Range(m_yaw);
        }	

        // Pitch - rotation around x axis
        public void Pitch(float amount)
        {
	        m_pitch += amount;
	        m_pitch = RestrictAngleTo360Range(m_pitch);
        }

        // Roll - rotation around z axis
        // Note: normally only used for aircraft type cameras rather than land based ones
        public void Roll(float amount) 
        {
	        m_roll += amount;
	        m_roll = RestrictAngleTo360Range(m_roll);
        }

        // Keep the angle in the range 0 to 360 (2*PI)
        public float RestrictAngleTo360Range(float angle)
        {
	        while(angle > 2 * Math.PI)
		        angle -= (float)(2 * Math.PI);

	        while(angle < 0)
		        angle += (float)(2 * Math.PI);

	        return angle;
        }

        public void Move()
        {

        }
    }
}
